"""
This module provides all combinations of the mixins provided by rest_framework.
All combinations follow the same format: {initials}Mixin.

The initials correspond to the following methods, in this specific order:
    - L: list()
    - R: retrieve()
    - C: create()
    - U: update()
    - P: partial_update()
    - D: delete()

So for example, to import a mixin that gives us list, retrieve and create,
we can do the following:

```
from rest_framework_mixins import LRCMixin
```
"""

from typing import Any

from django.views.generic import View
from rest_framework import mixins


class NoPutMixin:
    "A mixin that doesn't bring PUT along"

    http_method_names = [
        http_method_name
        for http_method_name in View.http_method_names
        if http_method_name != "put"
    ]


class NoPatchMixin:
    "A mixin that doesn't bring PATCH along"

    http_method_names = [
        http_method_name
        for http_method_name in View.http_method_names
        if http_method_name != "patch"
    ]


class LMixin(
    mixins.ListModelMixin,
):
    """
    A mixin that provides the following default action:

    - list()
    """


class RMixin(
    mixins.RetrieveModelMixin,
):
    """
    A mixin that provides the following default action:

    - retrieve()
    """


class CMixin(
    mixins.CreateModelMixin,
):
    """
    A mixin that provides the following default action:

    - create()
    """


class UMixin(
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default action:

    - update()
    """


class PMixin(
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default action:

    - partial_update()
    """


class DMixin(
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default action:

    - destroy()
    """


class LRMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    """


class LCMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    """


class LUMixin(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - update()
    """


class LPMixin(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - partial_update()
    """


class LDMixin(
    mixins.ListModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - destroy()
    """


class RCMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    """


class RUMixin(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - update()
    """


class RPMixin(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - partial_update()
    """


class RDMixin(
    mixins.RetrieveModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - destroy()
    """


class CUMixin(
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - update()
    """


class CPMixin(
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - partial_update()
    """


class CDMixin(
    mixins.CreateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - destroy()
    """


class UPMixin(
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - update()
    - partial_update()
    """


class UDMixin(
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - update()
    - destroy()
    """


class PDMixin(
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - partial_update()
    - destroy()
    """


class LRCMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    """


class LRUMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - update()
    """


class LRPMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - partial_update()
    """


class LRDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - destroy()
    """


class LCUMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - update()
    """


class LCPMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - partial_update()
    """


class LCDMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - destroy()
    """


class LUPMixin(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - update()
    - partial_update()
    """


class LUDMixin(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - update()
    - destroy()
    """


class LPDMixin(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - partial_update()
    - destroy()
    """


class RCUMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - update()
    """


class RCPMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - partial_update()
    """


class RCDMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - destroy()
    """


class RUPMixin(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - update()
    - partial_update()
    """


class RUDMixin(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - update()
    - destroy()
    """


class RPDMixin(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - partial_update()
    - destroy()
    """


class CUPMixin(
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - update()
    - partial_update()
    """


class CUDMixin(
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - update()
    - destroy()
    """


class CPDMixin(
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - partial_update()
    - destroy()
    """


class UPDMixin(
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - update()
    - partial_update()
    - destroy()
    """


class LRCUMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - update()
    """


class LRCPMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - partial_update()
    """


class LRCDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - destroy()
    """


class LRUPMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - update()
    - partial_update()
    """


class LRUDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - update()
    - destroy()
    """


class LRPDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - partial_update()
    - destroy()
    """


class LCUPMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - update()
    - partial_update()
    """


class LCUDMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - update()
    - destroy()
    """


class LCPDMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - partial_update()
    - destroy()
    """


class LUPDMixin(
    mixins.ListModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - update()
    - partial_update()
    - destroy()
    """


class RCUPMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - update()
    - partial_update()
    """


class RCUDMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - update()
    - destroy()
    """


class RCPDMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - partial_update()
    - destroy()
    """


class RUPDMixin(
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - update()
    - partial_update()
    - destroy()
    """


class CUPDMixin(
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - create()
    - update()
    - partial_update()
    - destroy()
    """


class LRCUPMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - update()
    - partial_update()
    """


class LRCUDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPatchMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - update()
    - destroy()
    """


class LRCPDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
    NoPutMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - partial_update()
    - destroy()
    """


class LRUPDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - update()
    - partial_update()
    - destroy()
    """


class LCUPDMixin(
    mixins.ListModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - create()
    - update()
    - partial_update()
    - destroy()
    """


class RCUPDMixin(
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - retrieve()
    - create()
    - update()
    - partial_update()
    - destroy()
    """


class LRCUPDMixin(
    mixins.ListModelMixin,
    mixins.RetrieveModelMixin,
    mixins.CreateModelMixin,
    mixins.UpdateModelMixin,
    mixins.DestroyModelMixin,
):
    """
    A mixin that provides the following default actions:

    - list()
    - retrieve()
    - create()
    - update()
    - partial_update()
    - destroy()
    """



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['rest_framework_mixins']

package_data = \
{'': ['*']}

install_requires = \
['django>=4.1,<5.0', 'djangorestframework']

entry_points = \
{'console_scripts': ['gen = rest_framework_mixins.generate:main']}

setup_kwargs = {
    'name': 'rest-framework-mixins',
    'version': '0.1.3',
    'description': 'A collection of DRF mixins combinations',
    'long_description': '# rest-framework-mixins\n\n## Installation\n\n`pip install rest_framework_mixins`\n\n## Usage\n\nThis package provides all combinations of the mixins provided by rest_framework.  \nAll combinations follow the same format: `{initials}Mixin.\n\nThe initials correspond to the following methods, in this specific order:\n\n- L: `list()`  \n- R: `retrieve()`  \n- C: `create()`  \n- U: `update()`  \n- P: `partial_update()`  \n- D: `delete()`  \n\nSo for example, to import a mixin that gives us list, retrieve and create,\nwe can do the following:\n\n```\nfrom rest_framework_mixins import LRCMixin\n\nclass CreateListRetrieveViewSet(LRCMixin, viewsets.GenericViewSet):\n    """\n    A viewset that provides `retrieve`, `create`, and `list` actions.\n\n    To use it, override the class and set the `.queryset` and\n    `.serializer_class` attributes.\n    """\n    pass\n```\n\n> Adapted from [DRF\'s documentation](https://www.django-rest-framework.org/api-guide/viewsets/#example_3)\n',
    'author': 'Xavier Francisco',
    'author_email': 'xavier.n.francisco@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/Qu4tro/rest_framework_mixins',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""
Script to programmatically generate all the mixin combinations
"""


import itertools
from collections.abc import Iterable, Iterator

import inflection
from black import FileMode, format_str

# Required due to the code in BASE_TEMPLATE
# pylint: disable=duplicate-code
BASE_TEMPLATE = """
'''
This module provides all combinations of the mixins provided by rest_framework.
All combinations follow the same format: {{initials}}Mixin.

The initials correspond to the following methods, in this specific order:
    - L: list()
    - R: retrieve()
    - C: create()
    - U: update()
    - P: partial_update()
    - D: delete()

So for example, to import a mixin that gives us list, retrieve and create,
we can do the following:

```
from rest_framework_mixins import LRCMixin
```
'''

from typing import Any

from django.views.generic import View
from rest_framework import mixins


class NoPutMixin:
    "A mixin that doesn't bring PUT along"

    http_method_names = [
        http_method_name
        for http_method_name in View.http_method_names
        if http_method_name != "put"
    ]


class NoPatchMixin:
    "A mixin that doesn't bring PATCH along"

    http_method_names = [
        http_method_name
        for http_method_name in View.http_method_names
        if http_method_name != "patch"
    ]


{mixins}
"""

CLASS_TEMPLATE = """class {name}({mixins}):
    '''
A mixin that provides the following default {action_noun}:

- {actions}
    '''
"""


def per_combination(combination: Iterable[str]) -> str:
    """
    Generate a string representation of a mixin combination class

        Args:
            combination: An iterable with action names

        Returns:
            A string representation of a mixin combination class
    """
    name = f"{''.join([action[0].upper() for action in combination])}Mixin"

    mixins: list[str] = []
    actions: list[str] = []
    for action_name in combination:
        actions.append(f"{inflection.underscore(action_name)}()")

        if action_name == "PartialUpdate":
            if "Update" in combination:
                continue
            action_name = "Update"

        mixins.append(f"mixins.{action_name}ModelMixin")

    if "PartialUpdate" not in combination and "Update" in combination:
        mixins.append("NoPatchMixin")
    if "Update" not in combination and "PartialUpdate" in combination:
        mixins.append("NoPutMixin")

    return CLASS_TEMPLATE.format(
        name=name,
        mixins=",".join(mixins) + ",",
        actions="\n- ".join(actions),
        action_noun="action" if len(actions) == 1 else "actions",
    )


def per_mixins(viewset_actions: list[str]) -> Iterator[str]:
    """
    Generate all mixin combinations for the given actions.

        Args:
            viewset_actions: The actions to generate the mixins for

        Returns:
            An iterator with each element being the string representation
            of a mixin combination class
    """
    for action_count in range(1, len(viewset_actions) + 1):
        for combination in itertools.combinations(viewset_actions, r=action_count):
            yield per_combination(combination)


def all_mixin_combinations() -> str:
    """
    Generates all mixin combinations for the actions provided by default by DRF.

        Returns:
            A black-formatted string with all the code
            required to be implanted onto __init__.py
    """
    code_per_class = per_mixins(
        ["List", "Retrieve", "Create", "Update", "PartialUpdate", "Destroy"]
    )
    code = BASE_TEMPLATE.format(mixins="\n".join(code_per_class))

    return format_str(code, mode=FileMode())


def main() -> None:
    "Prints to stdout the generated code"
    print(all_mixin_combinations(), end="")


if __name__ == "__main__":
    main()
